# Generated by ariadne-codegen on 2023-07-27 18:00
# Source: queries

from .async_base_client import AsyncBaseClient
from .generate_g_n_q_l import GenerateGNQL
from .get_c2s import GetC2s
from .get_i_ps import GetIPs
from .get_knocks import GetKnocks
from .get_noise_ranks import GetNoiseRanks
from .get_requests import GetRequests


def gql(q: str) -> str:
    return q


class Client(AsyncBaseClient):
    async def get_c2s(self) -> GetC2s:
        query = gql(
            """
            query getC2s {
              topC2s {
                c2s {
                  source_ip
                  hits
                  pervasiveness
                  c2_ips
                  c2_domains
                  payload
                }
              }
            }
            """
        )
        variables: dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return GetC2s.parse_obj(data)

    async def get_requests(self) -> GetRequests:
        query = gql(
            """
            query getRequests {
              topHTTPRequests {
                httpRequests {
                  path
                  request_count
                  source_ip_count
                  request_headers
                  pervasiveness
                }
              }
            }
            """
        )
        variables: dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return GetRequests.parse_obj(data)

    async def get_knocks(self, ip: str) -> GetKnocks:
        query = gql(
            """
            query getKnocks($ip: String!) {
              topKnocks(ip: $ip) {
                knock {
                  source_ip
                  headers
                  apps
                  emails
                  favicon_mmh3_128
                  favicon_mmh3_32
                  ips
                  knock_port
                  jarm
                  last_seen
                  last_crawled
                  links
                  title
                  tor_exit
                }
              }
            }
            """
        )
        variables: dict[str, object] = {"ip": ip}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return GetKnocks.parse_obj(data)

    async def generate_g_n_q_l(self, input: str) -> GenerateGNQL:
        query = gql(
            """
            query generateGNQL($input: String!) {
              generateGNQL(input: $input) {
                input_text
                queries
              }
            }
            """
        )
        variables: dict[str, object] = {"input": input}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return GenerateGNQL.parse_obj(data)

    async def get_noise_ranks(self, ip: str) -> GetNoiseRanks:
        query = gql(
            """
            query getNoiseRanks($ip: String!) {
              noiseRank(ip: $ip) {
                ips {
                  ip
                  noise_score
                  country_pervasiveness
                  payload_diversity
                  port_diversity
                  request_rate
                  sensor_pervasiveness
                }
              }
            }
            """
        )
        variables: dict[str, object] = {"ip": ip}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return GetNoiseRanks.parse_obj(data)

    async def get_i_ps(self) -> GetIPs:
        query = gql(
            """
            query getIPs {
              topPopularIPs {
                popularIPs {
                  ip
                  request_count
                  users_count
                }
              }
            }
            """
        )
        variables: dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return GetIPs.parse_obj(data)



# Generated by ariadne-codegen on 2023-07-27 18:00
# Source: queries

from typing import List

from pydantic import Field

from .base_model import BaseModel


class GetIPs(BaseModel):
    top_popular_i_ps: "GetIPsTopPopularIPs" = Field(alias="topPopularIPs")


class GetIPsTopPopularIPs(BaseModel):
    popular_i_ps: List["GetIPsTopPopularIPsPopularIPs"] = Field(alias="popularIPs")


class GetIPsTopPopularIPsPopularIPs(BaseModel):
    ip: str
    request_count: int
    users_count: int


GetIPs.update_forward_refs()
GetIPsTopPopularIPs.update_forward_refs()
GetIPsTopPopularIPsPopularIPs.update_forward_refs()



# Generated by ariadne-codegen on 2023-06-16 04:37
# Source: queries

from typing import List

from pydantic import Field

from .base_model import BaseModel


class NoiseRank(BaseModel):
    noise_rank: "NoiseRankNoiseRank" = Field(alias="noiseRank")


class NoiseRankNoiseRank(BaseModel):
    query_info: "NoiseRankNoiseRankQueryInfo" = Field(alias="queryInfo")
    ips: List["NoiseRankNoiseRankIps"]


class NoiseRankNoiseRankQueryInfo(BaseModel):
    results_available: int = Field(alias="resultsAvailable")
    results_limit: int = Field(alias="resultsLimit")


class NoiseRankNoiseRankIps(BaseModel):
    country_pervasiveness: str
    ip: str
    noise_score: int
    payload_diversity: str
    port_diversity: str
    request_rate: str
    sensor_pervasiveness: str


NoiseRank.update_forward_refs()
NoiseRankNoiseRank.update_forward_refs()
NoiseRankNoiseRankQueryInfo.update_forward_refs()
NoiseRankNoiseRankIps.update_forward_refs()



# Generated by ariadne-codegen on 2023-07-27 18:00
# Source: queries

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel


class GenerateGNQL(BaseModel):
    generate_g_n_q_l: "GenerateGNQLGenerateGNQL" = Field(alias="generateGNQL")


class GenerateGNQLGenerateGNQL(BaseModel):
    input_text: str
    queries: List[Optional[str]]


GenerateGNQL.update_forward_refs()
GenerateGNQLGenerateGNQL.update_forward_refs()



# Generated by ariadne-codegen on 2023-07-27 18:00
# Source: queries

from typing import List

from pydantic import Field

from .base_model import BaseModel


class GetRequests(BaseModel):
    top_h_t_t_p_requests: "GetRequestsTopHTTPRequests" = Field(alias="topHTTPRequests")


class GetRequestsTopHTTPRequests(BaseModel):
    http_requests: List["GetRequestsTopHTTPRequestsHttpRequests"] = Field(
        alias="httpRequests"
    )


class GetRequestsTopHTTPRequestsHttpRequests(BaseModel):
    path: str
    request_count: int
    source_ip_count: int
    request_headers: List[str]
    pervasiveness: int


GetRequests.update_forward_refs()
GetRequestsTopHTTPRequests.update_forward_refs()
GetRequestsTopHTTPRequestsHttpRequests.update_forward_refs()



# Generated by ariadne-codegen on 2023-06-16 04:37
# Source: queries

from typing import List

from pydantic import Field

from .base_model import BaseModel


class TopHTTPRequests(BaseModel):
    top_h_t_t_p_requests: "TopHTTPRequestsTopHTTPRequests" = Field(
        alias="topHTTPRequests"
    )


class TopHTTPRequestsTopHTTPRequests(BaseModel):
    query_info: "TopHTTPRequestsTopHTTPRequestsQueryInfo" = Field(alias="queryInfo")
    http_requests: List["TopHTTPRequestsTopHTTPRequestsHttpRequests"] = Field(
        alias="httpRequests"
    )


class TopHTTPRequestsTopHTTPRequestsQueryInfo(BaseModel):
    results_available: int = Field(alias="resultsAvailable")
    results_limit: int = Field(alias="resultsLimit")


class TopHTTPRequestsTopHTTPRequestsHttpRequests(BaseModel):
    method: str
    path: str
    pervasiveness: int
    request_count: int
    request_headers: List[str]
    source_ip_count: int


TopHTTPRequests.update_forward_refs()
TopHTTPRequestsTopHTTPRequests.update_forward_refs()
TopHTTPRequestsTopHTTPRequestsQueryInfo.update_forward_refs()
TopHTTPRequestsTopHTTPRequestsHttpRequests.update_forward_refs()



__author__ = "GreyNoise Intelligence"
__copyright__ = "Copyright, GreyNoise"
__credits__ = ["GreyNoise Intelligence"]
__license__ = "MIT"
__maintainer__ = "GreyNoise Intelligence"
__email__ = "labs@greynoise.io"
__status__ = "BETA"
__version__ = "0.1.7"



# Generated by ariadne-codegen on 2023-07-27 18:00
# Source: queries

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel


class GetC2s(BaseModel):
    top_c2s: "GetC2sTopC2s" = Field(alias="topC2s")


class GetC2sTopC2s(BaseModel):
    c2s: List["GetC2sTopC2sC2s"]


class GetC2sTopC2sC2s(BaseModel):
    source_ip: str
    hits: int
    pervasiveness: int
    c2_ips: List[Optional[str]]
    c2_domains: List[Optional[str]]
    payload: str


GetC2s.update_forward_refs()
GetC2sTopC2s.update_forward_refs()
GetC2sTopC2sC2s.update_forward_refs()



# Generated by ariadne-codegen on 2023-07-27 18:00
# Source: schemas/schema.graphql

from .base_model import BaseModel


class SpectaQLOption(BaseModel):
    key: str
    value: str


SpectaQLOption.update_forward_refs()



#!/usr/bin/env python3

import asyncio
import json
import jsonlines
import os

from greynoiselabs.__version__ import __version__
from greynoiselabs.api.client import Client
from greynoiselabs.cli.auth import login, authenticate
import typer
from typing_extensions import Annotated

AUTH0_DOMAIN = os.getenv("AUTH0_DOMAIN", default="greynoise2.auth0.com")
AUTH0_CLIENT_ID = os.getenv(
    "AUTH0_CLIENT_ID", default="IM8Old6x7WCr2wqVI0Cz3I0c4JPSR1gn"
)
ALGORITHMS = ["RS256"]
DEFAULT_TOKEN_FILE = "~/.greynoiselabs/token.json"

app = typer.Typer(no_args_is_help=True)

current_user = None
token_data = None
client = None


def initOutfile(outfile):
    if outfile != "":
        try:
            writer = jsonlines.open(outfile, mode="w")
            return writer
        except Exception as ex:
            print(f"unable to open {outfile} for writing: {ex}")
            raise typer.Abort()
    else:
        return None


def out(obj, outfile_writer):
    """
    Output the object as JSON
    """
    try:
        if outfile_writer:
            outfile_writer.write(obj.__dict__)
        else:
            print(json.dumps(obj.__dict__))
    except Exception as ex:
        print(f"unable to dump object {ex}")
        typer.Abort()


def new_client(id_token):
    return Client(
        "https://api.labs.greynoise.io/1/query", {"Authorization": f"Bearer {id_token}"}
    )


@app.command()
def version():
    """
    Return the version of the GreyNoise Labs CLI.
    """
    print(__version__)


@app.command()
def init(
    token: Annotated[
        str, typer.Option(help="Output location of the token file.")
    ] = DEFAULT_TOKEN_FILE
):
    """
    Initialize the client by authenticating with Auth0 and saving the token to a file.
    """
    global current_user, token_data, client
    token_data, current_user = authenticate(token)
    if token_data is None or current_user is None:
        run_init = typer.confirm(
            "You are not authenticated, would you like to do this now?"
        )
        if run_init:
            token_data, current_user = login(token)
            print("Authentication successful, please re-run your command")
            raise typer.Abort()
        else:
            print("You must authenticate to use this CLI")
            raise typer.Abort()
    if token_data is not None and current_user is not None:
        try:
            client = new_client(token_data["id_token"])
        except Exception as ex:
            print(f"Unable to create client {ex}")
            raise typer.Abort()
    else:
        print("Authentication failed, please try again")
        raise typer.Abort()


@app.command()
def c2s(output: Annotated[str, typer.Option(help="JSON output file location.")] = ""):
    """
    Return the top 1% of C2s ranked by pervasiveness.
    This data may be up to 4.5 hours old but covers the previous 24 hours.
    """
    init()
    writer = initOutfile(output)
    response = asyncio.run(client.get_c2s())
    for c2 in response.top_c2s.c2s:
        out(c2, writer)
    if writer:
        writer.close()


@app.command()
def knocks(
    ip: Annotated[str, typer.Argument(help="Specify the IP to retrieve.")] = "",
    output: Annotated[str, typer.Option(help="JSON output file location.")] = "",
):
    """
    Return the top 1% of Knock results by most recently seen.
    A Knock represents an IP observed by GreyNoise that we have scanned back.
    This data may be up to 12 hours old.
    This endpoint supports filtering by a single IP.
    """
    init()
    writer = initOutfile(output)
    response = asyncio.run(client.get_knocks(ip))
    for knock in response.top_knocks.knock:
        out(knock, writer)
    if writer:
        writer.close()


@app.command()
def popular_ips(
    output: Annotated[str, typer.Option(help="JSON output file location.")] = ""
):
    """
    Return the top 1% of IPs searched in GreyNoise.
    These results are ordered by the number of users observed over the last 7 days.
    This data may be up to 24 hours old.
    This also returns the user and request counts.
    This also returns a boolean if this IP was observed by GreyNoise sensors.
    """
    init()
    writer = initOutfile(output)
    response = asyncio.run(client.get_i_ps())
    for ip in response.top_popular_i_ps.popular_i_ps:
        out(ip, writer)
    if writer:
        writer.close()


@app.command()
def noise_rank(
    ip: Annotated[str, typer.Argument(help="Specify the IP to retrieve.")] = "",
    output: Annotated[str, typer.Option(help="JSON output file location.")] = "",
):
    """
    Return the top 1% of ranked IPs by noise score over the previous 7 days of traffic.
    This also returns the pervasiveness and diversity scores.
    This endpoint supports filtering by a single IP.
    """
    init()
    writer = initOutfile(output)
    response = asyncio.run(client.get_noise_ranks(ip))
    for ip in response.noise_rank.ips:
        out(ip, writer)
    if writer:
        writer.close()



# Generated by ariadne-codegen on 2023-07-27 18:00

from .async_base_client import AsyncBaseClient
from .base_model import BaseModel
from .client import Client
from .exceptions import (
    GraphQLClientError,
    GraphQLClientGraphQLError,
    GraphQLClientGraphQLMultiError,
    GraphQLClientHttpError,
    GraphQlClientInvalidResponseError,
)
from .generate_g_n_q_l import GenerateGNQL, GenerateGNQLGenerateGNQL
from .get_c2s import GetC2s, GetC2sTopC2s, GetC2sTopC2sC2s
from .get_i_ps import GetIPs, GetIPsTopPopularIPs, GetIPsTopPopularIPsPopularIPs
from .get_knocks import GetKnocks, GetKnocksTopKnocks, GetKnocksTopKnocksKnock
from .get_noise_ranks import (
    GetNoiseRanks,
    GetNoiseRanksNoiseRank,
    GetNoiseRanksNoiseRankIps,
)
from .get_requests import (
    GetRequests,
    GetRequestsTopHTTPRequests,
    GetRequestsTopHTTPRequestsHttpRequests,
)
from .input_types import SpectaQLOption

__all__ = [
    "AsyncBaseClient",
    "BaseModel",
    "Client",
    "GenerateGNQL",
    "GenerateGNQLGenerateGNQL",
    "GetC2s",
    "GetC2sTopC2s",
    "GetC2sTopC2sC2s",
    "GetIPs",
    "GetIPsTopPopularIPs",
    "GetIPsTopPopularIPsPopularIPs",
    "GetKnocks",
    "GetKnocksTopKnocks",
    "GetKnocksTopKnocksKnock",
    "GetNoiseRanks",
    "GetNoiseRanksNoiseRank",
    "GetNoiseRanksNoiseRankIps",
    "GetRequests",
    "GetRequestsTopHTTPRequests",
    "GetRequestsTopHTTPRequestsHttpRequests",
    "GraphQLClientError",
    "GraphQLClientGraphQLError",
    "GraphQLClientGraphQLMultiError",
    "GraphQLClientHttpError",
    "GraphQlClientInvalidResponseError",
    "SpectaQLOption",
]



# Generated by ariadne-codegen on 2023-06-16 04:37
# Source: queries

from typing import List

from pydantic import Field

from .base_model import BaseModel


class TopPopularIPs(BaseModel):
    top_popular_i_ps: "TopPopularIPsTopPopularIPs" = Field(alias="topPopularIPs")


class TopPopularIPsTopPopularIPs(BaseModel):
    query_info: "TopPopularIPsTopPopularIPsQueryInfo" = Field(alias="queryInfo")
    popular_i_ps: List["TopPopularIPsTopPopularIPsPopularIPs"] = Field(
        alias="popularIPs"
    )


class TopPopularIPsTopPopularIPsQueryInfo(BaseModel):
    results_available: int = Field(alias="resultsAvailable")
    results_limit: int = Field(alias="resultsLimit")


class TopPopularIPsTopPopularIPsPopularIPs(BaseModel):
    ip: str
    request_count: int
    users_count: int


TopPopularIPs.update_forward_refs()
TopPopularIPsTopPopularIPs.update_forward_refs()
TopPopularIPsTopPopularIPsQueryInfo.update_forward_refs()
TopPopularIPsTopPopularIPsPopularIPs.update_forward_refs()



# Generated by ariadne-codegen on 2023-07-27 18:00
# Source: queries

from typing import List

from pydantic import Field

from .base_model import BaseModel


class GetNoiseRanks(BaseModel):
    noise_rank: "GetNoiseRanksNoiseRank" = Field(alias="noiseRank")


class GetNoiseRanksNoiseRank(BaseModel):
    ips: List["GetNoiseRanksNoiseRankIps"]


class GetNoiseRanksNoiseRankIps(BaseModel):
    ip: str
    noise_score: int
    country_pervasiveness: str
    payload_diversity: str
    port_diversity: str
    request_rate: str
    sensor_pervasiveness: str


GetNoiseRanks.update_forward_refs()
GetNoiseRanksNoiseRank.update_forward_refs()
GetNoiseRanksNoiseRankIps.update_forward_refs()



#!/usr/bin/env python
"""GreyNoise Labs API client package."""
import os

from setuptools import find_packages, setup


def read(fname):
    """Read file and return its contents."""
    with open(os.path.join(os.path.dirname(__file__), fname)) as input_file:
        return input_file.read()


setup(
    name="greynoiselabs",
    version="0.1.7",
    description="Abstraction to interact with GreyNoise Labs GraphQL API.",
    url="https://api.labs.greynoise.io/",
    author="GreyNoise Intelligence",
    author_email="labs@greynoise.io",
    license="MIT",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    install_requires=read("requirements/common.txt").split("\n"),
    long_description=read("README.rst") + "\n\n" + read("CHANGELOG.md"),
    python_requires=">=3.0, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries",
    ],
    entry_points={"console_scripts": ["greynoiselabs = greynoiselabs.cli.main:app"]},
    zip_safe=False,
    keywords=["internet", "scanning", "threat intelligence", "security"],
    download_url="https://github.com/GreyNoise-Intelligence/greynoiselabs",
)



# Generated by ariadne-codegen on 2023-06-16 04:37
# Source: queries

from typing import Any, List

from pydantic import Field

from .base_model import BaseModel


class TopKnocks(BaseModel):
    top_knocks: "TopKnocksTopKnocks" = Field(alias="topKnocks")


class TopKnocksTopKnocks(BaseModel):
    query_info: "TopKnocksTopKnocksQueryInfo" = Field(alias="queryInfo")
    knock: List["TopKnocksTopKnocksKnock"]


class TopKnocksTopKnocksQueryInfo(BaseModel):
    results_available: int = Field(alias="resultsAvailable")
    results_limit: int = Field(alias="resultsLimit")


class TopKnocksTopKnocksKnock(BaseModel):
    tor_exit: bool
    title: str
    source_ip: str
    links: List[str]
    last_seen: Any
    last_crawled: Any
    knock_port: int
    jarm: str
    ips: List[str]
    headers: str
    favicon_mmh3_32: int
    favicon_mmh3_128: str
    emails: List[str]
    apps: str


TopKnocks.update_forward_refs()
TopKnocksTopKnocks.update_forward_refs()
TopKnocksTopKnocksQueryInfo.update_forward_refs()
TopKnocksTopKnocksKnock.update_forward_refs()



# Generated by ariadne-codegen on 2023-07-27 18:00
# Source: queries

from typing import Any, List

from pydantic import Field

from .base_model import BaseModel


class GetKnocks(BaseModel):
    top_knocks: "GetKnocksTopKnocks" = Field(alias="topKnocks")


class GetKnocksTopKnocks(BaseModel):
    knock: List["GetKnocksTopKnocksKnock"]


class GetKnocksTopKnocksKnock(BaseModel):
    source_ip: str
    headers: str
    apps: str
    emails: List[str]
    favicon_mmh3_128: str
    favicon_mmh3_32: int
    ips: List[str]
    knock_port: int
    jarm: str
    last_seen: Any
    last_crawled: Any
    links: List[str]
    title: str
    tor_exit: bool


GetKnocks.update_forward_refs()
GetKnocksTopKnocks.update_forward_refs()
GetKnocksTopKnocksKnock.update_forward_refs()



#!/usr/bin/env python3

import time
import json
import os
from requests_oauthlib import OAuth2Session
from auth0.authentication.token_verifier import (
    TokenVerifier,
    AsymmetricSignatureVerifier,
)
import jwt
import requests
import typer

AUTH0_DOMAIN = "greynoise2.auth0.com"
AUTH0_CLIENT_ID = "IM8Old6x7WCr2wqVI0Cz3I0c4JPSR1gn"
TOKEN_URL = f"https://{AUTH0_DOMAIN}/oauth/token"
JWKS_URL = f"https://{AUTH0_DOMAIN}/.well-known/jwks.json"
ISSUER_URL = f"https://{AUTH0_DOMAIN}/"
ALGORITHMS = ["RS256"]


def get_token_from_file(token_filename):
    """
    Gets the token from a file
    """
    try:
        fp = os.path.expanduser(token_filename)
        with open(fp, "r") as f:
            token_data = json.load(f)
            return token_data
    except Exception as ex:
        print(f"Unable to load token from {token_filename}, {ex}")
        return None


def token_saver(token_filename, token_data):
    """
    Saves the token to a file
    """
    try:
        fp = os.path.expanduser(token_filename)
        os.makedirs(os.path.dirname(fp), exist_ok=True)
        with open(fp, "w+") as f:
            json.dump(token_data, f)
    except Exception as ex:
        print(f"Unable to save token to {token_filename}, {ex}")
        raise typer.Abort()


def token_refresh(token_filename, token_data_in):
    """
    Refreshes the token
    """
    extra = {
        "client_id": AUTH0_CLIENT_ID,
    }
    try:
        client = OAuth2Session(AUTH0_CLIENT_ID, token=token_data_in)
        token_data = client.refresh_token(TOKEN_URL, **extra)
        token_saver(token_filename, token_data)
        return token_data
    except Exception as ex:
        print(f"Unable to refresh token {ex}.")
        return None


def validate_token(id_token):
    """
    Verify the token and its precedence

    :param id_token:
    """
    sv = AsymmetricSignatureVerifier(JWKS_URL)
    tv = TokenVerifier(
        signature_verifier=sv, issuer=ISSUER_URL, audience=AUTH0_CLIENT_ID
    )
    try:
        tv.verify(id_token)
        return True
    except Exception as ex:
        print(f"Unable to validate token {ex}.")
        return False


def init_device_flow():
    """
    Initiate Oauth 2.0 Device Authorization Flow with Auth0
    """
    authenticated = False
    device_code_payload = {
        "client_id": AUTH0_CLIENT_ID,
        "scope": "openid profile offline_access",
    }
    device_code_response = requests.post(
        "https://{}/oauth/device/code".format(AUTH0_DOMAIN), data=device_code_payload
    )

    if device_code_response.status_code != 200:
        print("Error generating the device code")
        raise typer.Exit(code=1)

    print("Device code successful")
    device_code_data = device_code_response.json()
    print("1. Please browse to: ", device_code_data["verification_uri_complete"])
    print("2. Verify the code matches: ", device_code_data["user_code"])

    token_payload = {
        "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
        "device_code": device_code_data["device_code"],
        "client_id": AUTH0_CLIENT_ID,
    }
    while not authenticated:
        print("Please click the link above and follow the instructions...")
        token_response = requests.post(TOKEN_URL, data=token_payload)
        token_data = token_response.json()
        if token_response.status_code == 200:
            # print('- Id Token: {}...'.format(token_data['id_token'][:10]))
            current_user = jwt.decode(
                token_data["id_token"],
                algorithms=ALGORITHMS,
                options={"verify_signature": False},
            )
            authenticated = True
        elif token_data["error"] not in ("authorization_pending", "slow_down"):
            print(token_data["error_description"])
            raise typer.Exit(code=1)
        else:
            time.sleep(device_code_data["interval"])
    return token_data, current_user


def authenticate(token_filename):
    """
    Checks for local token and validates it, if invalid, attempts to refresh it.
    """
    global token_data
    global current_user
    validated = False

    token_data = get_token_from_file(token_filename)
    if token_data:
        validated = validate_token(token_data["id_token"])
        if validated:
            current_user = jwt.decode(
                token_data["id_token"],
                algorithms=ALGORITHMS,
                options={"verify_signature": False},
            )
            return token_data, current_user
        else:
            # Attempt to refresh the token when possible
            # the one we have in the file is not valid.
            token_data = token_refresh(token_filename, token_data)
            validated = validate_token(token_data["id_token"])
            if validated:
                current_user = jwt.decode(
                    token_data["id_token"],
                    algorithms=ALGORITHMS,
                    options={"verify_signature": False},
                )
                return token_data, current_user
    else:
        return None, None


def login(token_filename):
    """
    Runs the device authorization flow and stores the user object in memory
    """

    global token_data
    global current_user
    validated = False
    token_data, current_user = init_device_flow()
    token_saver(token_filename, token_data)
    validated = validate_token(token_data["id_token"])
    if validated:
        current_user = jwt.decode(
            token_data["id_token"],
            algorithms=ALGORITHMS,
            options={"verify_signature": False},
        )
        return token_data, current_user
    else:
        print("Unable to obtain a valid token, please contact labs@greynoise.io.")
        typer.Abort()
